#include <iostream>
#include <vector>
#include <queue>
#include <set>
#include <algorithm>

using namespace std;

// Function to find all guarded warehouses
void findGuardedWarehouses(int n, int m, int k, vector<pair<int, int>> &roads, vector<pair<int, int>> &guards) {
    vector<vector<int>> graph(n + 1);
    for (const auto &road : roads) {
        int a = road.first, b = road.second;
        graph[a].push_back(b);
        graph[b].push_back(a);
    }

    set<int> guarded;

    for (const auto &guard : guards) {
        int startWarehouse = guard.first, endurance = guard.second;
        if (endurance == 0) {
            guarded.insert(startWarehouse);
            continue;
        }

        queue<pair<int, int>> q;
        vector<bool> visited(n + 1, false);

        q.push({startWarehouse, 0});
        visited[startWarehouse] = true;

        while (!q.empty()) {
            auto [currentWarehouse, distance] = q.front();
            q.pop();

            if (distance > endurance) continue;
            guarded.insert(currentWarehouse);

            for (int neighbor : graph[currentWarehouse]) {
                if (!visited[neighbor]) {
                    visited[neighbor] = true;
                    q.push({neighbor, distance + 1});
                }
            }
        }
    }

    vector<int> guardedWarehouses(guarded.begin(), guarded.end());
    sort(guardedWarehouses.begin(), guardedWarehouses.end());

    cout << guardedWarehouses.size() << endl;
    for (int warehouse : guardedWarehouses) {
        cout << warehouse << " ";
    }
    if (!guardedWarehouses.empty()) cout << endl;
}

int main() {
    int n, m, k;
    cin >> n >> m >> k;

    vector<pair<int, int>> roads(m);
    for (int i = 0; i < m; ++i) {
        cin >> roads[i].first >> roads[i].second;
    }

    vector<pair<int, int>> guards(k);
    for (int i = 0; i < k; ++i) {
        cin >> guards[i].first >> guards[i].second;
    }

    findGuardedWarehouses(n, m, k, roads, guards);

    return 0;
}