program GuardedWarehouses;

uses sysutils;

const
  MAX_N = 200000;
  MAX_M = 200000;

type
  TEdge = record
    target, next: LongInt;
  end;

var
  Head: array[1..MAX_N] of LongInt; // Список смежности
  Edges: array[1..2 * MAX_M] of TEdge;
  Distances: array[1..MAX_N] of LongInt;
  Guarded: array[1..MAX_N] of Boolean;
  Queue: array[1..MAX_N] of LongInt;
  n, m, k, EdgeCount: LongInt;

procedure AddEdge(a, b: LongInt);
begin
  Inc(EdgeCount);
  Edges[EdgeCount].target := b;
  Edges[EdgeCount].next := Head[a];
  Head[a] := EdgeCount;
end;

procedure BFS(startNode: LongInt; maxDistance: LongInt);
var
  front, back, currentNode, edgeIndex, targetNode, distance: LongInt;
begin
  front := 0;
  back := 1;
  Queue[back] := startNode;
  Distances[startNode] := 0;

  while front < back do
  begin
    Inc(front);
    currentNode := Queue[front];
    distance := Distances[currentNode];

    edgeIndex := Head[currentNode];
    while edgeIndex <> 0 do
    begin
      targetNode := Edges[edgeIndex].target;
      if (Distances[targetNode] > distance + 1) and (distance + 1 <= maxDistance) then
      begin
        Distances[targetNode] := distance + 1;
        Inc(back);
        Queue[back] := targetNode;
      end;
      edgeIndex := Edges[edgeIndex].next;
    end;
  end;
end;

var
  i, a, b, pi, hi, count: LongInt;
  guardedWarehouses: TStringList;
begin
  ReadLn(n, m, k);

  FillChar(Head, SizeOf(Head), 0);
  EdgeCount := 0;

  for i := 1 to m do
  begin
    ReadLn(a, b);
    AddEdge(a, b);
    AddEdge(b, a);
  end;

  FillChar(Guarded, SizeOf(Guarded), False);
  for i := 1 to k do
  begin
    ReadLn(pi, hi);
    FillChar(Distances, SizeOf(Distances), $7F);
    
    BFS(pi, hi);
    for a := 1 to n do
    begin
      if Distances[a] <= hi then
        Guarded[a] := True;
    end; 
  end;

  count := 0;
  guardedWarehouses := TStringList.Create;
  try
    for i := 1 to n do
    begin
      if Guarded[i] then
      begin
        Inc(count);
        guardedWarehouses.Add(IntToStr(i));
      end;
    end;
    
    WriteLn(count);
    if count > 0 then
      WriteLn(guardedWarehouses.CommaText.Replace(',', ' ', [rfReplaceAll]));
  finally
    guardedWarehouses.Free;
  end;
end.